!     Solves a Ricardian trade model with 2 countries under symmetry
!     Calibrates the model to a steady state with free trade and a level of technological trade cost
!     Households can save using claims to the capital stock
!     Capital is produced using a mixture of tradables and nontradables
!     Capital producer creates investment services purchased by households
 
       MODULE parameters_mod
       implicit none
       
       integer, parameter :: nkpts = 50     !k is individual wealth (state variable 1)
       integer, parameter :: nepts = 5      !e is productivity shock (state variable 2)
       integer, parameter :: nlpts = 200    
       integer, parameter :: nk2pts = 5000
       integer, parameter :: nlzpts = 500
       integer, parameter :: nread = 0      ! Read in value function and decision rules
       integer, parameter :: nread2 = 1    ! Read in guess at transition path
       integer, parameter :: ncapt = 400
       integer, parameter :: ncut = ncapt/10
       
       integer, parameter :: nghost = 1
       
       integer, parameter :: mxiter = 2500  ! Maximum iterations on value function in zeta2 (steady state code) 
       integer, parameter :: mxiter2 = 1000
       integer, parameter :: mxpol = 200
       integer, parameter :: iwsize = 50    
       integer, parameter :: nwsize = 500   
       integer, parameter :: nparam = 3                              
 
                                            !HH's PROBLEM (inputs for solver)
       integer, parameter :: nf = 1        
       integer, parameter :: neq = 0      
       integer, parameter :: neqn = 0       
       integer, parameter :: nineq = 1      
       integer, parameter :: nineqn = 0   
     
                                                        !MODEL PARAMETERS
                                    ! trade parameters
       real*8, parameter :: one = 1.0D+00               
       real*8, parameter :: zn = 1.0D+00
       real*8, parameter :: zx = 1.0D+00
       real*8, parameter :: hours = 0.3D+00
       real*8, parameter :: sigma = 2.0D+00   ! CRRA parameter
       real*8, parameter :: eta = 1.2887757567D+00     ! steepnes in productivity curves
       real*8, parameter :: alpha_t = 0.64D+00   ! share parameter in tradables production
       real*8, parameter :: alpha_n = 0.64D+00   ! share parameter in nontradables production
       real*8, parameter :: kappa =  0.563D+00  ! tradables share in capital production
       real*8, parameter :: delta = 0.05D+00
       real*8, parameter :: theta = 5.72D+00
       
       real*8, parameter :: coef1 = (one/(eta*(theta-one)))**(one/(theta-one))
       
                                            !GRIDS 
       real*8, dimension(nkpts)  :: xkpts   
       real*8, dimension(nk2pts) :: xk2pts  
       real*8, dimension(nepts)  :: epts   
       real*8, dimension(nlpts) :: lpts    
       real*8, dimension(nepts,nepts) :: pie 
       
       integer, parameter :: bsearch = 0
       
       real*8, parameter :: updater = 1.0D-01
       real*8, parameter :: updatex  = 1.0D-01
       real*8, parameter :: updatez = 5.0D-01
       real*8, parameter :: toler2 = 1.0D-05

    END MODULE parameters_mod
    
    
    MODULE states_mod
       use parameters_mod
       implicit none                                       
       real*8, dimension(nkpts,nepts) :: y2,optk2,optct2,optcn2,optx2,budg2,ev,evp 
       real*8, dimension(nkpts,nepts) :: optct2_init,optcn2_init,optk2_init
       real*8, dimension(nkpts,nepts) :: y2_fin,optct2_fin,optcn2_fin,optk2_fin,optx2_fin
       real*8, dimension(nkpts,nepts) :: ystar,ystarg1,ystarg2,ystarg3
       real*8, dimension(nk2pts,nepts) :: f0,f1,fstart                        
       real*8, dimension(nepts) :: invareps
       real*8, dimension(nkpts,nepts,nlpts) :: vhat,vhat2,evhat,evphat
       real*8, dimension(ncapt) :: wpath,wpathnew,ptpath,pnpath,pnpathnew,pxpath,biglpath,bigltpath,biglnpath,bighpath,bigkpath,bigknpath,bigktpath,bigctpath,bigcnpath &
                                   ,bigxpath,biginpath,bigitpath,bigytpath,bigynpath,taxpath,taxpathnew
       real*8, dimension(ncapt+1) :: rpath,rpathnew
       real*8 :: r,w,rtilde,wtilde,pt,pn,px,bigyn,bigyt,bigcn,bigct,bigk,bigkt,bigkn,bigl,biglt,bigln,bigh,bigit,bigin,aggl,nonsaves,avgincome,con_rat,con_rat25,pop_cnt25 !PRICES USED IN THE DECISION PROBLEM
       real*8 :: xkcur,ztilde,zhat,imp_share,gdp,tau,lower_omega,upper_omega,xmin,imports,imports_init,imports_fin
       real*8 :: rtilde_init,wtilde_init,px_init,pt_init,tau_init,tax_init,rtilde_fin,wtilde_fin,px_fin,pt_fin,tau_fin,r_fin,aggx_fin,trade_elas
       real*8 :: zeroproflab_t,aggx
       real*8 :: v_free
       integer :: neps,isave,nt,nk,ierr
       integer :: ind_init 
       integer :: rcheck                                   
       real*8 :: rlow,rhigh                              
       logical :: skip
    END MODULE states_mod

    
    MODULE interfaces_mod
       interface
          double precision function zeta2(x)    
             use parameters_mod
             use states_mod
             implicit none
             real*8, intent(in) :: x
          end function zeta2
       end interface
    END MODULE interfaces_mod
     
    MODULE calib_mod
       implicit none
       integer, parameter :: ncalib = 3   
       integer, parameter :: ncalib2 = 1  ! Reads calibration.out for last guess 
       real*8, parameter :: beta = 0.96046000D+00            
       real*8, parameter :: xkbor = 0.00D+00 
       real*8, parameter :: imptarg_init = 0.13D+00
       real*8, parameter :: imptarg_fin = 0.17D+00
       real*8 :: rat_05,rat_95,gamma,cbar
    END MODULE calib_mod
    
    PROGRAM MAIN
      use parameters_mod
      use states_mod
      use interfaces_mod
      use calib_mod
      implicit none
      real*8, parameter :: h = 0.9D+00
      integer :: i,j,k,jj,jk,kk,ttt,multpl,nsing,khi,klo,calibed,dumi
      real*8 :: dum,xkbar,fsum,rho,sigy,base,wgt,xkinc,x1,x2,zbrent,rstar1,idum,r1,r2,wcalc,zstep
      real*8 :: con_t,con_n,xkp,eusum,delt_t,delt_n
      real*8 :: rmxdif
      real*8, dimension(1) :: yval
      real*8, dimension(nkpts,nepts) :: welf
      real*8, dimension(ncalib) :: z,z2,d,fv,fv1,fv2,fv3,fv4
      real*8, dimension(ncalib,ncalib) :: jacobian,jacinv
      real*8, dimension(2) :: fv_small
      real*8, dimension(2,2) :: jacobian_small,jacinv_small
      
      real*8, dimension(nepts) :: invar
      real*8, dimension(nepts) :: pit
      
      interface
         double precision function welfare(x)
            use parameters_mod
            use states_mod
            implicit none
            real*8, intent(in) :: x
         end function welfare
      end interface
      
      calibed = .true.
      skip = .true.
  
      pn = one


      f1 = 0.0D+00
      f1(275,3) = one 
       
      fstart = f1
      
!      xkpts50.in is the coarse grid for state variable k (capital)
      open(2,file='xkpts50.in',form='formatted')
      do i = 1,nkpts
      read(2,390) xkpts(i)
390   format(f18.10)
      end do
      close(2)
 
      xkpts = xkpts + xkbor
      !building the fine grid
      xk2pts(1) = xkpts(1)
      xk2pts(nk2pts) = 100.00D+00!xkpts(nkpts)
       
      do i = 2,nk2pts-1
         xk2pts(i) = xk2pts(i-1)+(xk2pts(nk2pts)-xk2pts(1))/dble(real(nk2pts-1))
      end do

!     PRODUCTIVITY SHOCK PROCESS 
      
      sigy = 0.236D+00 
	   rho = 0.927D+00 
      
      
	   epts(1) = -sqrt(dble(nepts-1))*sigy/sqrt(one-rho*rho)
      epts(nepts) = -epts(1)
      do i = 2,nepts-1
         epts(i) = epts(i-1)+(epts(nepts)-epts(1))/dble(nepts-1)
      end do
      epts = exp(epts)
      epts = epts/epts(nepts/2+1)
      call rouwenhorst(rho,pie,nepts)
     
      open(4,file='pie.out',form='formatted')
      do i = 1,nepts
      write(4,831) pie(i,:)
      end do
      close(4)
      
      ! finds invariant distribution of productivity
      
      pit = one/dble(nepts)
      do i = 1,1000
        pit = matmul(pit,pie)
      end do
      
      invareps = pit
      
      do i = 1,nepts
         aggl = aggl + invareps(i)*epts(i)
      end do
      aggl = aggl*hours
      bigl = aggl
      
      open(4,file='epts.out',form='formatted')
      write(4,831) epts
831   format(5f18.10) 
      close(4)
      
      if (nread .eq. 1) then 
         print*, 'reading in prior value function'
         open(3,file='saving_ss.out',form='formatted')
           do i = 1,nkpts
             read(3,444) dum,y2(i,1:nepts),optk2(i,1:nepts),optx2(i,1:nepts),optct2(i,1:nepts),optcn2(i,1:nepts),dum,dum,dum,dum,dum
444          format(f18.10,5f18.10,5f18.10,5f18.10,5f18.10,5f18.10,5f18.10) 
           end do
         close(3)
       else
          do i = 1,nkpts
             do j = 1,nepts
                y2(i,j) = dlog(xkpts(i)+1.0D-02-xkbor) 
                optk2(i,j) = 0.3*xkpts(i)
                optct2(i,j) = 0.2D+00*0.70D+00*(xkpts(i)-xkbor)
                optcn2(i,j) = 0.4D+00*0.70D+00*(xkpts(i)-xkbor)
                optx2(i,j) =  0.4D+00*0.70D+00*(xkpts(i)-xkbor)
             end do
          end do
       endif 
         
       ind_init = 0
      
      if(calibed) then
         
         open(5,file='calibration.out',form='formatted')
         read(5,339) z2
         close(5)

         do ttt = 1,100
         
            print*, ttt, 'of 1000'
            write(6,571) z2
            
            z = z2
            
            call getf(z,fv)

            if(maxval(dabs(z2-z)) .lt. 1.0D-05) goto 2
         end do
      else
         print*, 'THIS SHOULD BE CALIBRATED.'
         pause
         if (ncalib2 .eq. 1) then   
            open(5,file='calibration.out',form='formatted')
            read(5,339) z2               
            close(5)
         else
            z2(1) = 1.0427231669D+00
            z2(2) = 0.2579239513D+00
            z2(3) = 0.1923994121D+00  
         endif
!         
          zstep = 1.0D-08
         do ttt = 1,100
      
            print*, ttt, 'of 1000'
            write(6,571) z2
571         format(3f10.6)
            z = z2
            
            call getf(z,fv)

            d = z
        
            d(1) = z(1)+zstep*z(1)
         
            call getf(d,fv1)
         
            d = z
         
            d(2) = z(2)+zstep*z(2)
         
            call getf(d,fv2)
         
            d = z 
         
            d(3) = z(3)+zstep*z(3)
         
            call getf(d,fv3)
         
            do i = 1,ncalib
               jacobian(i,1) = (fv1(i)-fv(i))/(zstep*z(1))
               jacobian(i,2) = (fv2(i)-fv(i))/(zstep*z(2))
               jacobian(i,3) = (fv3(i)-fv(i))/(zstep*z(3))
            end do
         
            call inv3(jacobian,jacinv,nsing)
         
            z2 = z-h*matmul(jacinv,fv)
            
            open(1,file='calib_full.out',form='formatted')
            write(1,339) z
            write(1,339) z2
            write(1,339) fv
            close(1)
         
            open(1,file='calibration.out',form='formatted')
            write(1,339) z2
339         format(3f18.10)
            close(1)
            print*, maxval(dabs(z2-z))
            if (maxval(dabs(z2-z)) .lt. 1.0D-05) goto 2 
            if (maxval(dabs(fv)) .lt. 1.0D-04) goto 2
         end do
      endif
      
2     print*, 'done with final steady state'
   
      open(1,file='parameters.in',form='formatted')
      write(1,340) tau,beta,gamma,cbar,xkbor,sigma,kappa,theta,alpha_t,alpha_n,delta,eta,lower_omega,upper_omega,zx,zn
340   format(16f18.10) 
      close(1)
            
      dum = zeta2(r) ! call one more time to get exact solution
      print*, 'actually done'
      
      r_fin = r
      aggx_fin = aggx
      tau_fin = z2(1)
      rtilde_fin = rtilde
      wtilde_fin = wtilde
      px_fin = px
      pt_fin = pt
      
      imports_fin = imports
      tau_fin = tau
      
      open(4,file='aggregates_fin.out',form='formatted')
         write(4,389) tau,r,pn,aggx,w,px,pt,bigk,bigyt,bigkt,biglt,bigct,bigit,bigyn,bigkn,bigln,bigcn,bigin
389      format(18f18.10)
      close(4)

      open(3,file='saving_ss_fin.out',form='formatted')
      do i = 1,nkpts
         write(3,456) xkpts(i),y2(i,1:nepts),optk2(i,1:nepts),optx2(i,1:nepts),optct2(i,1:nepts),optcn2(i,1:nepts),budg2(i,1:nepts)
456      format(f18.10,5f18.10,5f18.10,5f18.10,5f18.10,5f18.10,5f18.10)  
      end do
      close(3)
      
       open(5,file='saving_ss_fin.f1',form='formatted')
      do i = 1,nk2pts
         write(5,499) xk2pts(i),f1(i,:)
      end do
      close(5)
      
      y2_fin = y2
      optk2_fin = optk2
      optx2_fin = optx2
      optct2_fin = optct2
      optcn2_fin = optcn2
      
      print*, 'done through calibration'
      
      !!  FIND INITIAL STEADY STATE
      
      z2(1) = 1.1116998390D+00
      
      ind_init = 1
      
      zstep = 1.0D-07
      
      do ttt = 1,100
      
            print*, ttt, 'of 1000'
            print*, z2
            z = z2
            
            call getf(z,fv)

            if(maxval(dabs(fv(1:2))) .lt. 1.0D-07) goto 3

            d = z
        
            d(1) = z(1)+zstep*z(1)
         
            call getf(d,fv1)
            
            z2(1) = z(1)-h*fv(1)/((fv1(1)-fv(1))/1.0D-08)
            
            print*, maxval(dabs(z2-z))
            
            if(maxval(dabs(z2-z)) .lt. 1.0D-07) goto 3

      end do

3     print*, 'done with initial steady state'
      
      dum = zeta2(r)  ! call again to get exact solution
      
      rtilde_init = rtilde
      wtilde_init = wtilde
      px_init = px
      pt_init = pt
      imports_init = imports
      tau_init = tau
      
      open(4,file='aggregates_init.out',form='formatted')
         write(4,389) tau,r,pn,aggx,w,px,pt,bigk,bigyt,bigkt,biglt,bigct,bigit,bigyn,bigkn,bigln,bigcn,bigin
      close(4)

      optct2_init = optct2
      optcn2_init = optcn2
      optk2_init = optk2
      
      fstart = f1
      
      open(5,file='saving_ss.fstart',form='formatted')
      do i = 1,nk2pts
         write(5,499) xk2pts(i),fstart(i,:)
499     format(f18.10,5f18.10)
      end do
      close(5)
         
      print*, 'going to transition'
      
      if (nread2 .eq. 1) then 
       open(5,file='trans_sol.out',form='formatted')
       do i = 1,ncapt
         read(5,940) dumi,rpath(i),dum
      end do
      close(5)   
      else
         ! Assume a constant path equal to new steady state values
         rpath = r_fin
      endif
      
      do ttt = 1,mxiter2

         call trans

         rpath = (one-updater)*rpath+updater*rpathnew
         
         rmxdif = maxval(dabs(rpathnew(1:ncapt-ncut)-rpath(1:ncapt-ncut)))
         
         open(5,file='trans_sol.out',form='formatted')
         do i = 1,ncapt           
            write(5,940) i,rpath(i),rpathnew(i)
940         format(i5,2f18.10)
         end do
         close(5)
         
         if (rmxdif .lt. toler2) goto 12
         if(multpl(ttt,1) == 1) then 
            write(6,391) ttt,mxiter2,rmxdif
391         format(2i5,f14.9)
         endif
      end do

12     write(6,391) ttt,mxiter2,rmxdif
         
         open(5,file='transition_full.out',form='formatted')
         do i = 1,ncapt-ncut           
            write(5,939) i,rpath(i),wpath(i),ptpath(i),pxpath(i),bigxpath(i),bigkpath(i),bigytpath(i),bigynpath(i),bigktpath(i),bigknpath(i),bigltpath(i),biglnpath(i),bigctpath(i),bigcnpath(i),bigitpath(i),biginpath(i)
939         format(i5,16f18.10)
         end do
         close(5)
         
      print*, 'done with transition'
      
      !  THIS SECTION IS FOR COMPARING WELFARE BETWEEN STEADY STATES 
      
      !! Get the terminal steady state values for ghost households
      if(nghost .eq. 1) then
         call zetaghost(1,ystarg1)
         print*, 'done with ghost 1'
         call zetaghost(2,ystarg2) 
         print*, 'done with ghost 2'
         call zetaghost(3,ystarg3) 
         print*, 'done with ghost 3' 
      
      open(4,file='saving1.out',form='formatted')
      do i = 1,nkpts
         write(4,832) xkpts(i),ystarg1(i,:)
832      format(f18.10,5f18.12)
      end do
      close(4)
      
      open(4,file='saving2.out',form='formatted')
      do i = 1,nkpts
         write(4,832) xkpts(i),ystarg2(i,:)
      end do
      close(4)
      
      open(4,file='saving3.out',form='formatted')
      do i = 1,nkpts
         write(4,832) xkpts(i),ystarg3(i,:)
      end do
      close(4)

         print*, 'done with ghosts'
      endif
      
      lpts(1) = -0.21D+00
      lpts(nlpts) = 0.210D+00
          
      do i = 2,nlpts-1
         lpts(i) = lpts(i-1) + (lpts(nlpts)-lpts(1))/dble(real(nlpts-1))  
      end do
      
      vhat2 = 1.0D+00
      
      do ttt = 1,mxiter
         vhat = vhat2
         evhat = vhat
         do kk = 1,nlpts
            do jj = 1,nepts
               call dpchim(nkpts,xkpts,evhat(:,jj,kk),evphat(:,jj,kk),1,ierr)
            end do
            do j = 1,nepts
               neps = j
               do i = 1,nkpts
                  nk = i
                  con_t = optct2_init(i,j)
                  con_n = optcn2_init(i,j) 
                  xkp = optk2_init(i,j)
                  
                  eusum = 0.0D+00
                  do jj = 1,nepts
                     call dpchfe(nkpts,xkpts,evhat(:,jj,kk),evphat(:,jj,kk),1,skip,1,xkp,yval,ierr)
                     eusum = eusum + pie(neps,jj)*yval(1)
                  end do   
                  
                  if (sigma .eq. 1.0D+000) then
                     vhat2(i,j,kk) = gamma*dlog(((one+lpts(kk))*con_t))+(one-gamma)*dlog(((one+lpts(kk))*con_n)+cbar) + beta*eusum
                  else
                     vhat2(i,j,kk) = ((((one+lpts(kk))*con_t)**gamma*(((one+lpts(kk))*con_n)+cbar)**(one-gamma))**(one-sigma))/(one-sigma) + beta*eusum
                  endif
               end do
            end do
         end do
         if (maxval(dabs(vhat2-vhat)) < 1.0D-08) then
            goto 13
         endif
         if (multpl(ttt,100) .eq. 1) then
            print*, ttt,maxval(dabs(vhat2-vhat))
         endif
      end do     
      
      
13    open(1,file='trade.wel',form='formatted')
      do i = 1,nkpts
         nk = i
         do j = 1,nepts
            neps = j
            v_free = ystar(i,j) 
            wcalc = zbrent(welfare,lpts(1),lpts(nlpts),1.0D-08)
            welf(i,j) = wcalc
         end do 
         write(1,160) xkpts(i),welf(i,1:nepts)
160      format(f18.10,5f18.10)     
      end do
      close(1)

      if (nghost .eq. 1) then
      open(1,file='trade_ghost1.wel',form='formatted')
      do i = 1,nkpts
         nk = i
         do j = 1,nepts
            neps = j
            v_free = ystarg1(i,j) 
            wcalc = zbrent(welfare,lpts(1),lpts(nlpts),1.0D-08)
            welf(i,j) = wcalc
         end do 
         write(1,160) xkpts(i),welf(i,1:nepts)
      end do
      close(1)
      
      open(1,file='trade_ghost2.wel',form='formatted')
      do i = 1,nkpts
         nk = i
         do j = 1,nepts
            neps = j
            v_free = ystarg2(i,j)
            wcalc = zbrent(welfare,lpts(1),lpts(nlpts),1.0D-08)
            welf(i,j) = wcalc
         end do 
         write(1,160) xkpts(i),welf(i,1:nepts)
      end do
      close(1)
      
      open(1,file='trade_ghost3.wel',form='formatted')
      do i = 1,nkpts
         nk = i
         do j = 1,nepts
            neps = j
            v_free = ystarg3(i,j)
            wcalc = zbrent(welfare,lpts(1),lpts(nlpts),1.0D-08)
            welf(i,j) = wcalc
         end do 
         write(1,160) xkpts(i),welf(i,1:nepts)
      end do
      close(1) 
      endif

   END 

      subroutine trans
      use parameters_mod
      use states_mod
      use calib_mod
      use interfaces_mod
      implicit none
      integer, parameter :: maxa = 100
      real*8, parameter :: fac = 0.9D+00  
      real*8, dimension(nk2pts,nepts) :: vgrid,optkg,optxg,optctg,optcng
      real*8, dimension(nkpts,nepts,ncapt+1) :: y2t,optk2t,optx2t,optct2t,optcn2t,budg2t
      real*8 :: funca,eps,epsneq,udelta,bigbnd,ck,wnew,xknew,xknew2,xkhi,xklo,yval,yp,ydp,wght1,xk2inc,rinc,rstar,rwgt,rginc,xxx,aug
      real*8 :: bet,rate,fsum,zbrent,yt
      integer :: i,j,k,ii,jj,ttt,multpl,mode,iprint,miter,inform,npol,jk,ijk,khi,klo,jlo1
      integer, dimension(nk2pts,nepts) :: jlos1
      real*8, dimension(nk2pts,nepts) :: wghts1
      real*8, dimension(nwsize) :: work
      real*8, dimension(nparam) :: x,bl,bu
      real*8, dimension(nparam) :: x1,bl1,bu1
      real*8, dimension(nf+1) :: ff
      real*8, dimension(nineq+neq+1) :: g
      integer, dimension(nwsize) :: iw
      external func,dfunc,const,dconst,grobfd,grcnfd
      
      mode = 100
      iprint = 0
      miter = 500
      eps = 1.0D-12
      epsneq = 1.0D-12
      udelta = 0.0D+00
      bigbnd = 1.0D+10
 
      y2t(:,:,ncapt+1) = y2_fin
      optk2t(:,:,ncapt+1) = optk2_fin
      optx2t(:,:,ncapt+1) = optx2_fin
      optct2t(:,:,ncapt+1) = optct2_fin
      optcn2t(:,:,ncapt+1) = optcn2_fin
      tau = tau_fin
      
      upper_omega = min((dlog(tau)+eta)/(2.0D+00*eta),one)
      lower_omega = one-upper_omega

      ztilde = coef1*((tau**(one-theta))*(dexp(eta*(theta-one))-dexp(eta*upper_omega*(theta-one))) &
                       +dexp(eta*(theta-one))-dexp(eta*(one-upper_omega)*(theta-one)))**(one/(theta-one))  
      zhat = coef1*((tau**(-theta))*(dexp(eta*(theta-one))-dexp(eta*upper_omega*(theta-one))))**(one/(theta-one))
      
      do ttt = ncapt,1,-1

         y2 = y2t(:,:,ttt+1)
         
         r = rpath(ttt)
         
         w =  zn**(one/alpha_n)*alpha_n*pn**(one/alpha_n)*(r/(one-alpha_n))**((alpha_n-one)/alpha_n)
         wpath(ttt) = w
        
         pt = ((w/alpha_t)**alpha_t*(r/(one-alpha_t))**(one-alpha_t))/ztilde
         px = (one/zx)*(pt/kappa)**kappa*(pn/(one-kappa))**(one-kappa)  
         
         ptpath(ttt) = pt
         pxpath(ttt) = px
         
         rtilde = (r-px*delta)
         wtilde = w
         
         ev = y2
         
         do i = 1,nepts
            call dpchim(nkpts,xkpts,ev(:,i),evp(:,i),1,ierr)
         end do
         
         do j = 1,nepts
            neps = j
            do i = 1,nkpts
               nk = i
               xkcur = xkpts(i)
               
               bl(1) = xkbor ! bounds on x
               bl(2) = 1.0D-06      !lower bounds on ct
               bl(3) = 1.0D-06      !lower bound on cn
               bu(1) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours)/px-1.0D-06
               bu(2) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours-xkbor)/pt-1.0D-06
               bu(3) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours-xkbor)/pn-1.0D-06
              
               if (minval(bu-bl) .lt. 0.0D+00) then 
                  print*, 'something wrong in transition boundaries'
                  print*, bl
                  print*, bu
                  print*, r,w,xkcur
                  print*, px,pt,pn
                  pause
               endif
               
               x(1) = optk2t(i,j,ttt+1)
               x(2) = optct2t(i,j,ttt+1)
               x(3) = optcn2t(i,j,ttt+1)
               
               call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
                          epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,func,const,dfunc,dconst)
               !call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
               !           epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,func,const,grobfd,grcnfd)
               
                  optk2t(i,j,ttt) = x(1)
                  optct2t(i,j,ttt) = x(2)
                  optcn2t(i,j,ttt) = x(3)
                  optx2t(i,j,ttt) = x(1) - xkcur
                  y2t(i,j,ttt) = -ff(1)  
            end do
         end do      
            
         ystar = y2t(:,:,1)
      end do
         
      f1 = fstart
      
      do ttt = 1,ncapt
   
         f0 = f1
         f1 = 0.0D+00
         
         do j = 1,nepts
            do i = 1,nk2pts
               call hunt(xkpts,nkpts,xk2pts(i),jlo1)
               if (jlo1 .lt. 1) jlo1 = 1
               if (jlo1 .ge. nkpts) jlo1 = nkpts-1
               ck = (xk2pts(i)-xkpts(jlo1))/(xkpts(jlo1+1)-xkpts(jlo1))
               optkg(i,j) = ck*optk2t(jlo1+1,j,ttt)+(one-ck)*optk2t(jlo1,j,ttt)
               optxg(i,j) = ck*optx2t(jlo1+1,j,ttt)+(one-ck)*optx2t(jlo1,j,ttt)
               optctg(i,j) = ck*optct2t(jlo1+1,j,ttt)+(one-ck)*optct2t(jlo1,j,ttt)
               optcng(i,j) = ck*optcn2t(jlo1+1,j,ttt)+(one-ck)*optcn2t(jlo1,j,ttt)
            end do
         end do

         xk2inc = xk2pts(2)-xk2pts(1)
         
         do j = 1,nepts
            do i = 1,nk2pts
               call hunt(xk2pts,nk2pts,optkg(i,j),jlo1)
               if (jlo1 < 1) then
                  jlo1 = 1
                  wght1 = 1.0D+00
               elseif (jlo1 >= nk2pts) then
                  jlo1 = nk2pts-1
                  wght1 = 0.0D+00
               else   
                  wght1 = one-(optkg(i,j)-xk2pts(jlo1))/xk2inc  
               endif
                  wghts1(i,j) = wght1
                  jlos1(i,j) = jlo1
            end do
         end do

         do j = 1,nepts
            do i = 1,nk2pts
               wght1 = wghts1(i,j)
               jlo1 = jlos1(i,j)
               do jj = 1,nepts
                  f1(jlo1,jj) = wght1*pie(j,jj)*f0(i,j)+f1(jlo1,jj)
                  f1(jlo1+1,jj) = (one-wght1)*pie(j,jj)*f0(i,j)+f1(jlo1+1,jj)
               end do
            end do
         end do
           
         fsum = sum(f1)                 
         if (dabs(fsum-one) > 1.0D-06) then
            write(6,543) ttt,fsum
 543        format('does not integrate to 1: ',i10,f18.10)
            pause
         endif
         
         bigk = 0.0D+00
         bigct = 0.0D+00
         bigcn = 0.0D+00
         aggx = 0.0D+00
         
         do j = 1,nepts
            do i = 1,nk2pts
               bigk = bigk + xk2pts(i)*f0(i,j)
               bigct = bigct + optctg(i,j)*f0(i,j)
               bigcn = bigcn + optcng(i,j)*f0(i,j)
               aggx = aggx + (optxg(i,j)+delta*xk2pts(i))*f0(i,j)
            end do
         end do
         
         bigxpath(ttt) = aggx
         bigctpath(ttt) = bigct
         bigcnpath(ttt) = bigcn
         biglpath(ttt) = bigl
         bighpath(ttt) = bigh
         biginpath(ttt) = (aggx/zx)*((pt/pn)*(one-kappa)/kappa)**kappa
         bigitpath(ttt) = (aggx/zx)*((pt/pn)*(one-kappa)/kappa)**(kappa-one)
         bigytpath(ttt) = bigctpath(ttt)+bigitpath(ttt)
         bigynpath(ttt) = bigcnpath(ttt)+biginpath(ttt)
         
         imp_share = (ptpath(ttt)*bigytpath(ttt))/(ptpath(ttt)*bigytpath(ttt)+bigynpath(ttt))*(zhat/ztilde)**(theta-one)  

         bigkpath(ttt) = bigk
         bigktpath(ttt) = (one-alpha_t)*(ptpath(ttt)*bigytpath(ttt))/(rpath(ttt))
         bigknpath(ttt) = bigk - bigktpath(ttt)
         if (bigknpath(ttt) .le. 0.0D+00) then
            bigknpath(ttt) = 1.0D-02
         endif

         biglnpath(ttt) = (bigynpath(ttt)/zn)**(one/alpha_n)*bigknpath(ttt)**((alpha_n-one)/alpha_n)
         bigltpath(ttt) = bigl-biglnpath(ttt)
         
         if(bigltpath(ttt) .lt. 1.0D-08) bigltpath(ttt) = 1.0D-02
                          
         rpathnew(ttt) = zn*pn*(one-alpha_n)*(bigknpath(ttt)/biglnpath(ttt))**(-alpha_n)
         
         
         if (rpathnew(ttt) .le. 0.0D+00) then 
             rpathnew(ttt) = 1.0D-08
         endif
         
        
      end do

      return
      end

      double precision function zeta2(rate)
      use parameters_mod
      use states_mod
      use calib_mod
      implicit none
      integer, parameter :: maxa = 100
      real*8, parameter :: fac = 0.9D+00
      real*8, dimension(nkpts,nepts) :: y,optk,optcn,optct,optx,opth
      real*8 :: vmxdif,dmxdif,cmxdif,bmxdif,amxdif,funca,eps,epsneq,udelta,bigbnd,ck,wnew,xknew,xknew2,xkhi,xklo,yval,yp,ydp,xrat,rnew
      real*8 :: bet,rate
      integer :: i,j,ii,ttt,multpl,mode,iprint,miter,inform,npol,jk,ijk,khi,klo
      real*8, dimension(nwsize) :: work
      real*8, dimension(nparam) :: x,bl,bu
      real*8, dimension(nparam) :: x1,bl1,bu1
      real*8, dimension(nf+1) :: ff
      real*8, dimension(nineq+neq+1) :: g
      integer, dimension(nwsize) :: iw
      external func,dfunc,const,dconst,grobfd,grcnfd
      
      mode = 100
      iprint = 0
      miter = 500
      eps = 1.0D-12
      epsneq = 1.0D-12
      udelta = 0.0D+00
	   bigbnd = 1.0D+10
      
	   vmxdif = 100.0D+00
      
      r = rate
      
      w = zn**(one/alpha_n)*alpha_n*pn**(one/alpha_n)*(r/(one-alpha_n))**((alpha_n-one)/alpha_n)

      pt = ((w/alpha_t)**alpha_t*(r/(one-alpha_t))**(one-alpha_t))/ztilde
      px = (one/zx)*(pt/kappa)**kappa*(pn/(one-kappa))**(one-kappa)
      
      rtilde = r-px*delta
      wtilde = w
      !VALUE FUNCTION ITERATION (TO SOLVE THE HOUSEHOLD PROBLEM)
      do ttt = 1,mxiter

         y = y2
         optk = optk2
         optct = optct2
         optcn = optcn2
         optx = optx2
         
         if (vmxdif > 1.0D-01) then
            npol = 0
         else
            npol = 1
         endif

         ev = y
         do i = 1,nepts
            call dpchim(nkpts,xkpts,ev(:,i),evp(:,i),1,ierr)
         end do
         
         do j = 1,nepts             
            neps = j                
            do i = 1,nkpts          
               xkcur = xkpts(i)     
               bl(1) = xkbor 
               bl(2) = 1.0D-06    
               bl(3) = 1.0D-06   
               
               bu(1) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours)/px-1.0D-06
               bu(2) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours-xkbor)/pt-1.0D-06
               bu(3) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours-xkbor)/pn-1.0D-06
               
               
               if (minval(bu-bl) .lt. 0.0D+00) then 
                     print*, 'something wrong with boundaries in zeta2'
                     print*, bl
                     print*, bu
                     print*, r,w,xkcur
                     print*, px,pt,pn
                     pause
                  endif
               
               x(1) = optk(i,j)     
               x(2) = optct(i,j)    
               x(3) = optcn(i,j)
               
               call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
                          epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,func,const,dfunc,dconst) !grobfd,grcnfd)
                         
               optk2(i,j) = x(1)    
               optct2(i,j) = x(2)
               optcn2(i,j) = x(3)
               
               budg2(i,j) = px*x(1)+pt*x(2)+pn*x(3)-wtilde*epts(j)*hours-rtilde*xkcur-px*xkcur
               y2(i,j) = -ff(1)     
            end do
         end do
	   
	     if (npol == 0) goto 2 !(Howard's improvement algorithm) 
         do ii = 1,mxpol
            ev = y2
            do i = 1,nepts
               call dpchim(nkpts,xkpts,ev(:,i),evp(:,i),1,ierr)
            end do
            do j = 1,nepts
               neps = j
               do i = 1,nkpts
                  xkcur = xkpts(i)
                  x(1) = optk2(i,j)
                  x(2) = optct2(i,j)
                  x(3) = optcn2(i,j)
                  y2(i,j) = funca(x)
               end do
            end do
         end do

         !calculating some sup norms
 2       vmxdif = maxval(dabs(y2-y))
         dmxdif = maxval(dabs(optk2-optk))
         cmxdif = maxval(dabs(optct2-optct))
         bmxdif = maxval(dabs(optcn2-optcn))
         if (vmxdif < 1.0D-08 .and. dmxdif < 1.0D-04 .and. cmxdif < 1.0D-04 .and. bmxdif < 1.0D-04  .and. ttt > 2) goto 8
         if (multpl(ttt,500) == 1) then
            write(6,888) ttt,mxiter,vmxdif,dmxdif,cmxdif,bmxdif
888         format(2i5,4f15.10)
         endif
      end do
      
8     continue

      do i = 1,nkpts
         do j = 1,nepts
            optx2(i,j) = optk2(i,j)-xkpts(i)
         end do
      end do
      
      open(3,file='saving_ss.out',form='formatted')
      do i = 1,nkpts
         write(3,456) xkpts(i),y2(i,1:nepts),optk2(i,1:nepts),optx2(i,1:nepts),optct2(i,1:nepts),optcn2(i,1:nepts),budg2(i,1:nepts)
456      format(f18.10,5f18.10,5f18.10,5f18.10,5f18.10,5f18.10,5f18.10)  
      end do
      close(3)
      
 7984 format('computing invariant distribution')
      
      call calcf
      
      open(22,file='saving_ss.f1',form='formatted')
      do i = 1,nk2pts
         write(22,499) xk2pts(i),f1(i,1:nepts)
499      format(f18.10,5f18.10) 
      end do
      close(22)
	   
      rnew = zn*pn*(one-alpha_n)*(bigkn/bigln)**(-alpha_n)

	   zeta2 =  rnew - r
  
      write(6,299) r,rnew,zeta2 
299   format(3f11.5)

      return
      end

      double precision function funca(x)
      use parameters_mod
      use states_mod
      use calib_mod
      implicit none
      real*8, intent(in), dimension(nparam) :: x
      real*8 :: ct,cn,xinv,utilc,eusum
      real*8, dimension(1) :: xkp,yval,yp
      integer :: i

      xkp(1) = x(1)
      ct = x(2)
      cn = x(3)
      
      if (sigma .eq. 1.0D+00) then
         utilc = gamma*dlog(ct)+(one-gamma)*dlog(cn+cbar)
      else
         utilc = ((ct**gamma*(cn+cbar)**(one-gamma))**(one-sigma))/(one-sigma)
      endif

      eusum = 0.0D+00
      
      do i = 1,nepts
         call dpchfe(nkpts,xkpts,ev(:,i),evp(:,i),1,skip,1,xkp,yval,ierr)
         eusum = eusum + pie(neps,i)*yval(1)
      end do
      
      funca = utilc+beta*eusum
      
      return
      end
      
      subroutine func(np,j,x,fj)
      use parameters_mod
      use states_mod
      use calib_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: fj
      real*8 :: ct,cn,xinv,utilc,eusum
      real*8, dimension(1) :: xkp,yval,yp
      integer :: i
      
      xkp(1) = x(1)
      ct = x(2)
      cn = x(3)
      
      if (sigma .eq. 1.0D+00) then
         utilc = gamma*dlog(ct)+(one-gamma)*dlog(cn+cbar)
      else
         utilc = ((ct**gamma*(cn+cbar)**(one-gamma))**(one-sigma))/(one-sigma)
      endif
      
      eusum = 0.0D+00
      
      do i = 1,nepts
         call dpchfe(nkpts,xkpts,ev(:,i),evp(:,i),1,skip,1,xkp,yval,ierr)
         eusum = eusum + pie(neps,i)*yval(1)
      end do
      
      fj = -utilc-beta*eusum
      
      if (bsearch .eq. 1) then
         print*, 'func'
         print*, x
         print*, utilc,eusum
         print*, -fj
         print*, '------------'
         pause
      endif
      
      return
      end

      subroutine dfunc(np,j,x,fj,dummy)
      use parameters_mod
      use states_mod
      use calib_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: fj
      real*8 :: ct,cn,xinv,utilc,eusum,duc,cagg
      real*8, dimension(1) :: xkp,yval,yp
      integer :: i
      external dummy
      
      xkp(1) = x(1)
      ct = x(2)
      cn = x(3)
      
      cagg = ct**gamma*(cn+cbar)**(one-gamma)
      duc = cagg**(-sigma)
      eusum = 0.0D+00
      
      do i = 1,nepts
         call dpchfd(nkpts,xkpts,ev(:,i),evp(:,i),1,skip,1,xkp,yval,yp,ierr)
         eusum = eusum + pie(neps,i)*yp(1)
      end do
      
      if (sigma .eq. 1.0D+00) then
         fj(1) = -beta*eusum
         fj(2) = -gamma/ct
         fj(3) = -(one-gamma)/(cn+cbar)
      else
         fj(1) = -beta*eusum
         fj(2) = -gamma*duc*(ct**(gamma-one)*(cn+cbar)**(one-gamma))
         fj(3) = -(one-gamma)*duc*(ct**gamma*(cn+cbar)**(-gamma))
      endif
         
      if (bsearch .eq. 1) then
         print*, 'dfunc'
         print*, -fj
         pause
         print*, '-----------------'
      endif
      
      return
      end

      subroutine const(np,j,x,gj)
      use parameters_mod
      use states_mod
      use calib_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out) :: gj
      real*8 :: xkp,xinv,ct,cn
      
      xkp = x(1)
      ct = x(2)
      cn = x(3)
      
      gj = pt*ct+pn*cn+px*xkp-rtilde*xkcur-wtilde*hours*epts(neps)-px*xkcur
      
      if (bsearch .eq. 1) then
         print*, 'const'
         print*, x
         print*, gj
         print*, pt*ct+pn*cn+px*xkp,-rtilde*xkcur-wtilde*hours*epts(neps)-px*xkcur
         print*, '-----------------'
         pause
      endif
      
      return
      end
                  
      subroutine dconst(np,j,x,gj,dummy)
      use parameters_mod
      use states_mod
      use calib_mod
      implicit none
      integer, intent(in) :: np,j
      real*8, intent(in), dimension(np) :: x
      real*8, intent(out), dimension(np) :: gj
      external dummy
      
      gj(1) = px
      gj(2) = pt
      gj(3) = pn
      
      !if (bsearch .eq. 1) then
      !   print*, 'dconst'
      !   print*, gj
      !   pause
      !   print*, '-----------------'
      !endif
      
      return
      end

      integer function multpl(i,m)
      implicit none
      integer, intent(in) :: i,m
      real*8, parameter :: toler = 1.0D-12
      real*8 :: remain

      remain = dble(real(i))/dble(real(m))-i/m

      if (remain <= toler) then
         multpl = 1
      else
         multpl = 0
      endif

      return
      end

      subroutine hunt(xx,n,x,jlo)
      implicit none
      integer, intent(in) :: n
      real*8, intent(in), dimension(n) :: xx
      logical :: ascnd
      real*8,  intent(in) :: x
      integer, intent(inout) :: jlo
      integer :: jhi,inc,jm

      ascnd = xx(n) >= xx(1)
      if (jlo <= 0 .or. jlo > n) then
         jlo = 0
         jhi = n+1
         goto 3
      endif

      inc = 1
      
      if (x >= xx(jlo) .eqv. ascnd) then
 1       jhi = jlo+inc
         if (jhi > n) then
            jhi = n+1
         elseif (x >= xx(jhi) .eqv. ascnd) then
            jlo = jhi
            inc = inc+inc
            goto 1
         endif
      else
         jhi = jlo
 2       jlo = jhi-inc
         if (jlo < 1) then
            jlo = 0
         elseif (x < xx(jlo) .eqv. ascnd) then
            jhi = jlo
            inc = inc+inc
            goto 2
         endif
      endif
 3    if (jhi-jlo == 1) then
         if (x == xx(n)) jlo = n-1
         if (x == xx(1)) jlo = 1
         return
      endif
      jm = (jhi+jlo)/2
      if (x >= xx(jm) .eqv. ascnd) then
         jlo = jm
      else
         jhi = jm
      endif
      goto 3

      end

      subroutine calcf
      use parameters_mod
      use states_mod
      use calib_mod
      implicit none
      
      integer, parameter :: mxit = 100000
      real*8 :: xkinc,pinc,wght1,fmxdif,fsum,vark,sk,xkurtk,sumx,sumy,xint,ginik,giniy,ginie,emean,ymean,yval,yp,ydp,rnew,wnew,b,ck,con_sum
      real*8 :: pop_cnt0,pop_cnt1
      integer :: i,j,jk,k,ttt,jlo1,nhpts,icount,khi,klo,multpl,npop
      real*8, dimension(nlzpts) :: pct
      real*8, dimension(nk2pts) :: fdist
      real*8, dimension(nk2pts,nepts) :: wghts1
      real*8, dimension(nk2pts,nepts) :: optkg,optxg,optctg,optcng,vgrid
      integer, dimension(nk2pts,nepts) :: jlos1
      real*8, dimension(nk2pts) :: xpt,ypt,yddp,aa,bb,cc,rr
      real*8, dimension(nk2pts*nepts) :: earn,ftemp,xpt2,ypt2,aa2,bb2,cc2,rr2,ydp2,yinc
      real*8, dimension(nlzpts) :: xlrz

      jk = 1

      do i = 1,nk2pts
         call hunt(xkpts,nkpts,xk2pts(i),jk) 
         if (jk .lt. 1) jk = 1
         if (jk .ge. nkpts) jk = nkpts-1
         ck = (xk2pts(i)-xkpts(jk))/(xkpts(jk+1)-xkpts(jk))  
         if (ck .gt. 1.0D+00 .or. ck .lt. 0.0D+00) then
            print*, 'ck is messed up', ck
            pause
         endif
         optkg(i,:) = (one-ck)*optk2(jk,:)+ck*optk2(jk+1,:)
         optxg(i,:) = (one-ck)*optx2(jk,:)+ck*optx2(jk+1,:)
         optctg(i,:) = (one-ck)*optct2(jk,:)+ck*optct2(jk+1,:)
         optcng(i,:) = (one-ck)*optcn2(jk,:)+ck*optcn2(jk+1,:)
         vgrid(i,:) = (one-ck)*y2(jk,:)+ck*y2(jk+1,:)
      end do
     
     xkinc = xk2pts(2)-xk2pts(1)

      jlo1 = 1
      do i = 1,nk2pts
         do j = 1,nepts
               call hunt(xk2pts,nk2pts,optkg(i,j),jlo1)
               if (jlo1 < 1) then
                  jlo1 = 1
                  wght1 = 1.0D+00
               elseif (jlo1 >= nk2pts) then
                  jlo1 = nk2pts-1
                  wght1 = 0.0D+00
               else   
                  wght1 = one-(optkg(i,j)-xk2pts(jlo1))/xkinc   !weight/share going to lower grid point
               endif
           wghts1(i,j) = wght1
	        jlos1(i,j) = jlo1
           if (wght1 .lt. 0.0D+00 .or. wght1 .gt. 1.0D+00) then
            print*, 'wgth1 is messed up',wght1
            pause
           endif
	     end do
      end do
      
      f1 = fstart
      
      do ttt = 1,mxit

         f0 = f1       
	      f1 = 0.0D+00   
         
         do i = 1,nk2pts
            do j = 1,nepts
               jlo1 = jlos1(i,j)
		         wght1 = wghts1(i,j)
		         do k = 1,nepts
                 f1(jlo1,k) = pie(j,k)*wght1*f0(i,j)+f1(jlo1,k)
                 f1(jlo1+1,k) = pie(j,k)*(one-wght1)*f0(i,j)+f1(jlo1+1,k)
               end do
           end do
         end do

         fmxdif = maxval(dabs(f1-f0))   
         
         fsum = sum(f1)               
         if (dabs(fsum-one) > 1.0D-06) then
            write(6,543) ttt,fsum
 543        format('does not integrate to 1: ',i10,f18.10)
            pause
         endif

         if (fmxdif <= 1.0D-12) goto 1
         
      end do

      
     
      !CALCULATING SOME STATISTICS FROM THE INVARIANT DISTRIBUTION f1
 1    bigk = 0.0D+00
      aggx = 0.0D+00
      bigct = 0.0D+00
      bigcn = 0.0D+00
      nonsaves = 0.0D+00
      avgincome = 0.0D+00
      con_rat = 0.0D+00
      con_rat25 = 0.0D+00
      con_sum = 0.0D+00
      
      pop_cnt1 = 0.0D+00
      pop_cnt0 = 0.0D+00
      pop_cnt25 = 0.0D+00
      npop = 0
      rat_05 = 0.0D+00
      rat_95 = 0.0D+00
      
      nonsaves = sum(f1(1,:))
      
      do i = 1,nk2pts 
         bigk = bigk+xk2pts(i)*sum(f1(i,:))
         pop_cnt1 = pop_cnt0 + sum(f1(i,:))
         
         if(pop_cnt1 .gt. 0.75D+00) then
            pop_cnt25 = sum(f1(i,:))+pop_cnt25
            do j = 1,nepts
            con_rat25 = con_rat25 + f1(i,j)*(pt*optctg(i,j)/(pt*optctg(i,j)+pn*optcng(i,j)))
            end do
         endif

         pop_cnt0 = pop_cnt1
          
         do j = 1,nepts
            aggx  = aggx  + (optxg(i,j)+delta*xk2pts(i))*f1(i,j) 
            bigct = bigct + optctg(i,j)*f1(i,j)   
            bigcn = bigcn + optcng(i,j)*f1(i,j)   
            avgincome = avgincome + (w*epts(j)*hours+(r-delta*px)*xk2pts(i))*f1(i,j)
            if (optcng(i,j) .gt. 1.0D-06) then
            con_rat = con_rat + (pt*optctg(i,j)/(pt*optctg(i,j)+pn*optcng(i,j)))*f1(i,j)
            con_sum = con_sum + f1(i,j)
            endif
         end do
      end do
      
      con_rat25 = con_rat25/pop_cnt25
      
      bigin = (aggx/zx)*((pt/pn)*(one-kappa)/kappa)**kappa
      bigit = (aggx/zx)*((pt/pn)*(one-kappa)/kappa)**(kappa-one)
     
      bigyt = bigct+bigit
      bigyn = bigcn+bigin
      
      gdp = pt*bigyt+pn*bigyn
      imports = pt*bigyt*(zhat/ztilde)**(theta-one)
      imp_share = imports/gdp
      
      if(ind_init .eq. 1) then
         trade_elas = ((imports_fin-imports)/((imports_fin+imports)))/((tau_fin-tau)/(tau_fin+tau))
      endif
      

      
      bigkt = (one-alpha_t)*(pt*bigyt)/r
      bigkn = bigk - bigkt

      if(bigkn .le. 0.0D+00) bigkn = 1.0D-02
      
      bigln = (bigyn/zn)**(one/alpha_n)*bigkn**((alpha_n-one)/alpha_n)
      if (bigln .lt. 0.0D+00) bigln = 1.0D-02
      
      biglt = aggl-bigln
      
      if (biglt .lt. 0.0D+00) biglt = 1.0D-02
      
      return
      end

      subroutine integr(xa,ya,y2a,n,xint)
      use parameters_mod
      implicit none
      integer, intent(in) :: n
      real*8, intent(in), dimension(n) :: xa,ya,y2a
      real*8, intent(out) :: xint
      real*8 :: one24,sum,diff
      integer :: i

      one24 = one/24.0D+00
      sum = 0.0D+00
      do i = 1,n-1
         diff = xa(i+1)-xa(i)
         sum = sum+diff*((ya(i)+ya(i+1))/2.0D+00-one24*diff*diff*(y2a(i)+y2a(i+1)))
      end do
      xint = sum

      return
      end

      double precision function zbrent(func,x1,x2,tol)
      implicit none
      real*8 :: x1,x2,tol,zip,fa,fb,a,b,c,fc,d,e,tol1,xm,s,p,q,r
      integer :: iter
      real*8, parameter :: eps = 3.0D-08
      integer, parameter :: itmax = 200 
      interface
         double precision function func(x)
         implicit none
         real*8, intent(in) :: x
         end function func
      end interface

      a = x1
      b = x2
      fa = func(a)
      fb = func(b)
      zip = 0.0D+00

      if ((fa > zip .and. fb > zip) .or. (fa < zip .and. fb < zip)) then
        print*, 'root must be bracketed'
        print*, a,fa
        print*, b,fb
        pause
      endif

      c = b
      fc = fb
      do iter = 1,itmax
         if ((fb > zip .and. fc > zip) .or. (fb < zip .and. fc < zip)) then
            c = a
            fc = fa
            d = b-a
            e = d
         endif
         if (dabs(fc) < dabs(fb)) then
            a = b
            b = c
            c = a
            fa = fb
            fb = fc
            fc = fa
         endif
         tol1 = 2.0D+00*eps*dabs(b)+0.5D+00*tol
         xm = 0.5D+00*(c-b)
         if (dabs(xm) <= tol1 .or. fb == zip) then
            zbrent = b
            return
         endif
         if (dabs(e) >= tol1 .and. dabs(fa) > dabs(fb)) then
            s = fb/fa
            if (a == c) then
               p = 2.0D+00*xm*s
               q = 1.0D+00-s
            else
               q = fa/fc
               r = fb/fc
               p = s*(2.0D+00*xm*q*(q-r)-(b-a)*(r-1.0D+00))
               q = (q-1.0D+00)*(r-1.0D+00)*(s-1.0D+00)
            endif
            if (p > zip) q = -q
            p = dabs(p)
            if (2.0D+00*p < dmin1(3.0D+00*xm*q-dabs(tol1*q),dabs(e*q))) then
               e = d
               d = p/q
            else
               d = xm
               e = d
            endif
         else
            d = xm
            e = d
         endif
         a = b
         fa = fb
         if (dabs(d) > tol1) then
            b = b+d
         else
            b = b+sign(tol1,xm)
         endif
         fb = func(b)
      end do

      pause 'zbrent exceeding maximum iterations'
      zbrent = b

      return
      end
      

	subroutine sort2(n,arr,brr)
	implicit none
	integer, intent(in) :: n
	real*8, dimension(n) :: arr,brr
	integer, parameter :: m = 7
	integer, parameter :: nstack = 50
	integer, dimension(nstack) :: istack
    integer :: jstack,l,ir,i,j,k
    real*8 :: a,b,temp

	jstack = 0
	l = 1
	ir = n
 1    if (ir-l < m) then
         do j = l+1,ir
	      a = arr(j)
	      b = brr(j)
	      do i = j-1,l,-1
	         if (arr(i) <= a) goto 2
	         arr(i+1) = arr(i)
	         brr(i+1) = brr(i)
          end do
            i = l-1
 2          arr(i+1) = a
            brr(i+1) = b
         end do
         if (jstack == 0) return
	   ir = istack(jstack)
	   l = istack(jstack-1)
	   jstack = jstack-2
	else
	   k = (l+ir)/2
	   temp = arr(k)
	   arr(k) = arr(l+1)
	   arr(l+1) = temp
	   temp = brr(k)
	   brr(k) = brr(l+1)
	   brr(l+1) = temp
	   if (arr(l) > arr(ir)) then
	      temp = arr(l)
	      arr(l) = arr(ir)
	      arr(ir) = temp
	      temp = brr(l)
	      brr(l) = brr(ir)
	      brr(ir) = temp
	   endif
	   if (arr(l+1) > arr(ir)) then
	      temp = arr(l+1)
	      arr(l+1) = arr(ir)
	      arr(ir) = temp
	      temp = brr(l+1)
	      brr(l+1) = brr(ir)
	      brr(ir) = temp
	   endif
	   if (arr(l) > arr(l+1)) then
	      temp = arr(l)
	      arr(l) = arr(l+1)
	      arr(l+1) = temp
	      temp = brr(l)
	      brr(l) = brr(l+1)
	      brr(l+1) = temp
	   endif
	   i = l+1
	   j = ir
	   a = arr(l+1)
	   b = brr(l+1)
 3       continue
            i = i+1
	   if (arr(i) < a) goto 3
 4       continue
            j = j-1
	   if (arr(j) > a) goto 4
	   if (j < i) goto 5
	   temp = arr(i)
	   arr(i) = arr(j)
	   arr(j) = temp
	   temp = brr(i)
	   brr(i) = brr(j)
	   brr(j) = temp
	   goto 3
 5       arr(l+1) = arr(j)
         arr(j) = a
	   brr(l+1) = brr(j)
         brr(j) = b
	   jstack = jstack+2
	   if (jstack > nstack) pause 'nstack too small in sort2'
	   if (ir-i+1 >= j-l) then
	      istack(jstack) = ir
	      istack(jstack-1) = i
	      ir = j-1
	   else
	      istack(jstack) = j-1
	      istack(jstack-1) = l
	      l = i
	   endif
	endif
	goto 1

	end
	
      subroutine getf(x,f)
      use parameters_mod
      use states_mod
      use calib_mod
      use interfaces_mod
      implicit none
      integer, parameter :: nrpts = 20
      real*8, dimension(nrpts) :: rpts,rerrors
      real*8, dimension(ncalib), intent(in) :: x
      real*8, dimension(ncalib), intent(out) :: f
      real*8 :: zbrent,rhat1,rhat2,rinc,dum
      integer :: i,rind
      
      tau = x(1)
      gamma = x(2)
      cbar = x(3)

      upper_omega = min((dlog(tau)+eta)/(2.0D+00*eta),one)

      lower_omega = one-upper_omega
      
      ztilde = coef1*((tau**(one-theta))*(dexp(eta*(theta-one))-dexp(eta*upper_omega*(theta-one))) &
                       +dexp(eta*(theta-one))-dexp(eta*(one-upper_omega)*(theta-one)))**(one/(theta-one))  
      zhat = coef1*((tau**(-theta))*(dexp(eta*(theta-one))-dexp(eta*upper_omega*(theta-one))))**(one/(theta-one))                   
  
      if (ind_init .eq. 1) then
         rhat1 = 0.0810D+00
         rhat2 = 0.0895D+00
      else
         rhat1 = 0.07855D+00
         rhat2 = 0.0895D+00
      endif
      
      r = zbrent(zeta2,rhat1,rhat2,1.0D-14)
      
      if (ind_init .eq. 1) then
        f(1) = imp_share/imptarg_init-1.0D+00
      else 
        f(1) = imp_share/imptarg_fin-1.0D+00
      endif
      f(2) = con_rat/0.35233D+00-1.0D+00
      f(3) = con_rat25/0.31250D+00 -1.0D+00
      
      
      if (ind_init .eq. 0) then
         print*, '       x'
         write(6,334) x
         print*, '       relative  error'
         write(6,334) f 
         print*, '       stats'
         print*, '    import share         avg c_t share     top 25% c_t share' 
         write(6,334) imp_share,con_rat,con_rat25
         print*,'        p*K/Y'     
         write(6,333) px*bigk/gdp 
334      format(3f18.10) 
333      format(f18.10)
      else
         print*, '       x'
         write(6,333)   x(1)
         print*, '       relative  error'
         write(6,333)  f(1)
         print*, '       stats'
         print*, 'import share = ', imp_share
         print*, 'trade_elas = ', trade_elas
      endif

      return
      end
   
	  double precision function xnormpdf(x,xmu,s2)
	  implicit none
	  real*8, parameter :: pi = 3.14159265D+00
	  real*8 :: x,xmu,s2

	  xnormpdf = dexp(-(x-xmu)*(x-xmu)/(2.0D+00*s2))/dsqrt(2.0D+00*pi*s2)

	  return
	  end

	  subroutine gaussnorm(x,w,n,xmu,s2)
	  implicit none
	  integer :: n
 	  real*8, parameter :: pi = 3.14159265D+00
 	  real*8 :: s2,xmu
 	  real*8, dimension(n) :: x0,w0,w,x

	  call gausshermite(x0,w0,n)
	  x = x0*dsqrt(2.0D+00*s2)+xmu
	  w = w0/dsqrt(pi)

	  return
	  end 
     
     subroutine gausshermite(x,w,n)
     implicit none
     integer, parameter :: maxit = 100
     real*8, parameter :: eps = 3.0D-14
     real*8, parameter :: pim4 = 0.7511255444649425D+00
     integer :: m,n,i,j,iter
     real*8 :: z,p1,p2,p3,drj,z1,pp
     real*8, dimension(n) :: x,w,y
     
     m = floor(dble(real(n+1)))/2
     do i = 1,m
      if (i == 1) then
         z = dsqrt(dble(real(2*n+1))-1.85575D+00*dble(real(2*n+1))**(-0.16667D+00))
	   elseif (i == 2) then
	      z = z-1.14D+00*(dble(real(n))**0.426D+00)/z
	   elseif (i == 3) then
	      z = 1.86D+00*z-0.86D+00*x(1)
	   elseif (i == 4) then
	      z = 1.91D+00*z-0.91D+00*x(2)
	   else
	      z = 2.0D+00*z-x(i-2)
	   endif
      do iter = 1,maxit
	      p1 = pim4
	      p2 = 0.0D+00
	      do j = 1,n
	         p3 = p2
	         p2 = p1
	         drj = dble(real(j))
	         p1 = z*dsqrt(2.0D+00/drj)*p2-dsqrt((drj-1.0D+00)/drj)*p3
          end do
          pp = dsqrt(2.0D+00*dble(real(n)))*p2
	      z1 = z
	      z = z1-p1/pp
	      if (dabs(z-z1) < eps) goto 1
       end do
 1     if (iter > maxit) pause 'error'
       x(i) = z
	   x(n+1-i) = -z
	   w(i) = 2.0D+00/(pp*pp)
	   w(n+1-i) = w(i)
    end do

    j = 0
    do i = n,1,-1
	   j = j+1
	   y(j) = x(i)
    end do
    x = y
    
    return
   end

    double precision function welfare(wel)
    use parameters_mod
    use states_mod
    implicit none
    real*8, intent(in) :: wel
    integer :: jl
    real*8 :: cl,term
      
    jl = 1
    call hunt(lpts,nlpts,wel,jl)
    if (jl < 1) jl = 1
    if (jl >= nlpts) jl = nlpts-1
    cl = dabs((wel-lpts(jl))/(lpts(jl+1)-lpts(jl)))
    term = (one-cl)*vhat2(nk,neps,jl)+cl*vhat2(nk,neps,jl+1)
    
    welfare = term-v_free
    
    return 
    end
    
    subroutine zetaghost(gtype,ystarg)
    use parameters_mod
    use states_mod
      use calib_mod
      implicit none
      integer, parameter :: maxa = 100
      real*8, parameter :: fac = 0.9D+00
      real*8, dimension(nkpts,nepts) :: y,optk,optcn,optct,optx,opth
      real*8, dimension(nkpts,nepts) :: y2g,optk2g,optcn2g,optct2g,optx2g,opth2g
      real*8, dimension(nkpts,nepts,ncapt+1) :: y2tg,optk2tg,optcn2tg,optct2tg,optx2tg,opth2tg
      real*8, dimension(nkpts,nepts), intent(out) :: ystarg
      real*8 :: vmxdif,dmxdif,cmxdif,bmxdif,amxdif,funca,eps,epsneq,udelta,bigbnd,ck,wnew,xknew,xknew2,xkhi,xklo,yval,yp,ydp
      real*8 :: wage
      integer :: i,j,ii,ttt,multpl,mode,iprint,miter,inform,npol,jk,ijk,khi,klo,gtype
      real*8, dimension(nwsize) :: work
      real*8, dimension(nparam) :: x,bl,bu
      real*8, dimension(nparam) :: x1,bl1,bu1
      real*8, dimension(nf+1) :: ff
      real*8, dimension(nineq+neq+1) :: g
      integer, dimension(nwsize) :: iw
      external func,dfunc,const,dconst,grobfd,grcnfd
      
      mode = 100
      iprint = 0
      miter = 500
      eps = 1.0D-12
      epsneq = 1.0D-12
      udelta = 0.0D+00
	   bigbnd = 1.0D+10
      
	   vmxdif = 100.0D+00
      
      if (gtype .eq. 1) then
         rtilde = rtilde_fin
         wtilde = wtilde_fin
         px = px_init
         pt = pt_init
      elseif(gtype .eq. 2) then
         rtilde = rtilde_init
         wtilde = wtilde_init
         px = px_fin
         pt = pt_init
      elseif(gtype .eq. 3) then
         rtilde = rtilde_init
         wtilde = wtilde_init
         px = px_init
         pt = pt_fin
      endif

      y2g = 0.0D+00
      optk2g = optk2
      optct2g = optct2
      optcn2g = optcn2

!VALUE FUNCTION ITERATION (TO SOLVE THE HOUSEHOLD PROBLEM)
      do ttt = 1,mxiter

         y = y2g
         optk = optk2g
         optct = optct2g
         optcn = optcn2g
         
         if (vmxdif > 1.0D-01) then
            npol = 0
         else
            npol = 1
         endif

         !npol = 0
         
         ev = y
         do i = 1,nepts
            call dpchim(nkpts,xkpts,ev(:,i),evp(:,i),1,ierr)
         end do
         
         do j = 1,nepts            
            neps = j                
            do i = 1,nkpts          
               xkcur = xkpts(i)     
               bl(1) = xkbor 
               bl(2) = 1.0D-06     
               bl(3) = 1.0D-06    
               
               bu(1) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours)/px-1.0D-06
               bu(2) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours-xkbor)/pt-1.0D-06
               bu(3) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours-xkbor)/pn-1.0D-06
               

               x(1) = optk(i,j)     
               x(2) = optct(i,j)    
               x(3) = optcn(i,j)

               call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
                          epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,func,const,dfunc,dconst) !grobfd,grcnfd)
                 
               optk2g(i,j) = x(1)    
               optct2g(i,j) = x(2)    
               optcn2g(i,j) = x(3)
               
               if (dabs(px*x(1)+pt*x(2)+x(3)-wtilde*epts(j)*hours-rtilde*xkcur-px*xkcur) .gt. 1.0D-06) then 
                  print*, 'budget not clearing for ghost household at, ', xkpts(i),epts(j)
                  print*, px*x(1)+pt*x(2)+x(3)-wtilde*epts(j)*hours-rtilde*xkcur-px*xkcur
                  print*, x
                  print*, r,px,pt
                  print*, w
               endif
               
               y2g(i,j) = -ff(1)     !value function

            end do
         end do
	   
	     if (npol == 0) goto 2  !(Howard's improvement algorithm) 
         do ii = 1,mxpol
            ev = y2g
            do i = 1,nepts
               call dpchim(nkpts,xkpts,ev(:,i),evp(:,i),1,ierr)
            end do
            do j = 1,nepts
               neps = j
               do i = 1,nkpts
                  xkcur = xkpts(i)
                  x(1) = optk2g(i,j)
                  x(2) = optct2g(i,j)
                  x(3) = optcn2g(i,j)
                  y2g(i,j) = funca(x)
               end do
            end do
         end do

         !calculating some sup norms
 2       vmxdif = maxval(dabs(y2g-y))
         dmxdif = maxval(dabs(optk2g-optk))
         cmxdif = maxval(dabs(optct2g-optct))
         bmxdif = maxval(dabs(optcn2g-optcn))
         
         if (vmxdif < 1.0D-08 .and. dmxdif < 1.0D-04 .and. cmxdif < 1.0D-04 .and. bmxdif < 1.0D-04 .and. ttt > 2) goto 8

         if (multpl(ttt,500) == 1) then
            write(6,888) ttt,mxiter,vmxdif,dmxdif,cmxdif,bmxdif
888         format(2i5,4f12.6)
         endif
      end do
         
8     continue
      
     ! Solve backward along transition

      y2tg(:,:,ncapt+1) = y2g
      optk2tg(:,:,ncapt+1) = optk2g
      optct2tg(:,:,ncapt+1) = optct2g
      optcn2tg(:,:,ncapt+1) = optcn2g
      
      do ttt = ncapt,1,-1
      
         y2g = y2tg(:,:,ttt+1)
         
         if (gtype .eq. 1) then
            rtilde = (rpath(ttt)-delta*pxpath(ttt))    
            wtilde = wpath(ttt)
            px = px_init
            pt = pt_init
         elseif (gtype .eq. 2) then
            rtilde = rtilde_init   
            wtilde = wtilde_init
            px = pxpath(ttt)
            pt = pt_init
         elseif (gtype .eq. 3) then
            rtilde = rtilde_init   
            wtilde = wtilde_init
            px = px_init
            pt = ptpath(ttt)
         endif 
         
         ev = y2g
         
         do i = 1,nepts
            call dpchim(nkpts,xkpts,ev(:,i),evp(:,i),1,ierr)
         end do
         
         do j = 1,nepts
            neps = j
            do i = 1,nkpts
               nk = i
               xkcur = xkpts(i)
               
               bl(1) = xkbor 
               bl(2) = 1.0D-06      
               bl(3) = 1.0D-06     
               
               bu(1) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours)/px-1.0D-06
               bu(2) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours-xkbor)/pt-1.0D-06
               bu(3) = (rtilde*xkcur+px*xkcur+wtilde*epts(j)*0.9999D+00*hours-xkbor)/pn-1.0D-06

               if (minval(bu-bl) .lt. 0.0D+00) then 
                  print*, 'something wrong in transition boundaries'
                  print*, bl
                  print*, bu
                  pause
               endif
            
               x(1) = optk2tg(i,j,ttt+1)
               x(2) = optct2tg(i,j,ttt+1)
               x(3) = optcn2tg(i,j,ttt+1)
               
                  call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
                          epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,func,const,dfunc,dconst)
               !call ffsqp(nparam,nf,nineqn,nineq,neqn,neq,mode,iprint,miter,inform,bigbnd,eps, &
               !           epsneq,udelta,bl,bu,x,ff,g,iw,iwsize,work,nwsize,func,const,grobfd,grcnfd)
               
                  optk2tg(i,j,ttt) = x(1)
                  optct2tg(i,j,ttt) = x(2)
                  optcn2tg(i,j,ttt) = x(3)

                  y2tg(i,j,ttt) = -ff(1)      
            end do
         end do      
            
         ystarg = y2tg(:,:,1)
         
      end do

      
      return
      end